<?php

namespace App\Api\V1\Middleware;

use App\Core\Database;
use App\Services\AuthService;
use App\Api\V1\Responses\ApiResponse;

/**
 * Middleware d'authentification pour les routes API.
 */
class AuthMiddleware
{
    private $authService;

    public function __construct()
    {
        $this->authService = new AuthService(Database::getInstance());
    }

    /**
     * Vérifie l'authentification pour les routes protégées.
     */
    public function handle()
    {
        // Vérifier si l'utilisateur est connecté via session
        if ($this->authService->isLoggedIn()) {
            return true;
        }

        // Vérifier le token Bearer (pour les appels API)
        $authHeader = $_SERVER['HTTP_AUTHORIZATION'] ?? '';
        if (preg_match('/Bearer\s+(.*)$/i', $authHeader, $matches)) {
            $token = $matches[1];
            if ($this->validateToken($token)) {
                return true;
            }
        }

        ApiResponse::unauthorized('Authentication required');
        return false;
    }

    /**
     * Validation simple du token (à remplacer par JWT en production).
     */
    private function validateToken($token)
    {
        try {
            $decoded = base64_decode($token);
            $parts = explode(':', $decoded);
            
            if (count($parts) === 3) {
                $userId = $parts[0];
                $timestamp = $parts[1];
                $signature = $parts[2];
                
                // Vérifier la signature
                $expectedSignature = base64_encode($userId . ':' . $timestamp . ':' . SESSION_SECRET);
                if ($signature === SESSION_SECRET) {
                    // Vérifier que le token n'est pas trop ancien (24h)
                    if ((time() - $timestamp) < 86400) {
                        // Reconstituer la session utilisateur
                        $this->restoreUserSession($userId);
                        return true;
                    }
                }
            }
        } catch (Exception $e) {
            // Token invalide
        }
        
        return false;
    }

    /**
     * Restaure la session utilisateur à partir de l'ID.
     */
    private function restoreUserSession($userId)
    {
        $db = Database::getInstance();
        $stmt = $db->prepare("SELECT id, username, role FROM users WHERE id = ?");
        $stmt->execute([$userId]);
        $user = $stmt->fetch();
        
        if ($user) {
            $_SESSION['user_id'] = $user['id'];
            $_SESSION['username'] = $user['username'];
            $_SESSION['role'] = $user['role'];
        }
    }
}